<?php

namespace App\Helpers;

use App\Models\Order;
use App\Models\SystemSetting;
use App\Repositories\CommonRepo;
use Carbon\Carbon;
use Request;
use File;

class FormatHelper
{
    /**
     * Set active class for routes
     *
     * @param string|array $path path or path array
     * @param string $active active class name. default value is active.
     * @return string
     */
    public static function set_active($path, $active = 'active')
    {
        if (is_array($path)) {
            return in_array(Request::path(), $path) ? 'active' : '';
        }

        return (Request::is($path) && !Request::is($path . '/*')) ? $active : '';
    }

    public static function set_same_active($path, $active = 'same')
    {
        if (is_array($path)) {
            return in_array(Request::path(), $path) ? 'same' : '';
        }

        return (Request::is($path) && !Request::is($path . '/*')) ? $active : '';
    }

    public static function collapsed_active($path, $active = 'collapsed')
    {
        if (is_array($path)) {
            return in_array(Request::path(), $path) ? 'collapsed' : '';
        }

        return (Request::is($path) && !Request::is($path . '/*')) ? $active : '';
    }

    public static function child_active($path, $active = 'child_active')
    {
        if (is_array($path)) {
            return in_array(Request::path(), $path) ? 'child_active' : '';
        }

        return (Request::is($path) && !Request::is($path . '/*')) ? $active : '';
    }

    /**
     * Set image url
     *
     * @param string|array $path path or path array
     * @param string $active active class name. default value is active.
     * @return string
     */
    public static function set_image_url($path)
    {
        if (File::exists(public_path('/') . $path)) {
            return $path;
        }

        return asset('back/img/no_image.jpg');
    }


    public static function getPercentageValue($goal, $raised)
    {
        if ($goal == 0) {
            $percentage = 0;
        } else {
            $percentage = round(($raised / $goal) * 100);
        }
        return $percentage;
    }

    public static function randNumberGenerator($length)
    {
        return substr(number_format(time() * rand(), 0, '', ''), 0, $length);
    }

    public static function loginButtonText()
    {
        $layers = CommonRepo::getLayers();

        if ($layers[2]->status == 1
            || $layers[3]->status == 1
            || $layers[4]->status == 1
            || $layers[1]->status == 1) {
            return 'Next';
        }
        $icon = "<i class=\"ft-unlock\"></i>";
        return $icon.' Sign in';
    }

    public static function stepTwoButtonText()
    {
        $layers = CommonRepo::getLayers();

        if ($layers[4]->status == 1) {
            return 'Next';
        }
        $icon = "<i class=\"ft-unlock\"></i>";
        return $icon.' Sign in';
    }

    public static function checkStepTwo($layers)
    {
        if($layers[1]->status == 1
            || $layers[2]->status == 1
            || $layers[3]->status == 1){
                return true;
        }
    }

    public static function checkStepThree($layers)
    {
        if($layers[4]->status == 1){
            return true;
        }
    }

    public static function getLayers()
    {
        $layers = CommonRepo::getLayers();

        return [
            'ssl' => $layers[0]->status,
            'mac' => $layers[1]->status,
            'ip_address' => $layers[2]->status,
            'pin' => $layers[3]->status,
            'dynamic_code' => $layers[4]->status,
            'captcha' => $layers[5]->status,
        ];
    }

    public static function checkSecure()
    {
        if(Request::isSecure()){
            return true;
        }

        return false;

    }

    public static function getNotifiCount()
    {
        $result = Order::select('id')
            ->where('status', 1)
            //->where('is_deleted', '<>', 1)
            ->get();

        if(!empty($result) && count($result) > 0){
            return count($result);
        }

        return false;
    }

    public static function getDate($dateTime){
        return Carbon::createFromFormat('Y-m-d H:i:s', $dateTime)->format('Y-m-d');
    }

    public static function settingInfo(){
        return SystemSetting::first();
    }

}