<?php

namespace App\Http\Controllers\Admin\Security\Auth;

use App\Helpers\FormatHelper;
use App\Repositories\CommonRepo;
use App\Repositories\UserRepo;
use App\Traits\CommonTrait;
use App\Traits\UserLoginTrait;
use App\Traits\UserTrait;
use BlakeGardner\MacAddress;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Contracts\Hashing\Hasher as HasherContract;
use Illuminate\Support\Facades\Auth;
use Spatie\SslCertificate\SslCertificate;

class LoginController extends Controller
{
    use UserLoginTrait, UserTrait;

    private $userRepo;
    private $certificate;

    public function __construct(HasherContract $hasher)
    {
        $this->userRepo = new UserRepo();
        $this->hasher = $hasher;
        $layers = CommonRepo::getLayers();
        if ($layers[0]->status == 1) {
            //$this->certificate = SslCertificate::createForHostName('a-platforms.com');
        }

    }

    public function showLoginForm(Request $request)
    {
        if (Auth::check()) {
            return redirect('/');
        }

        $text = FormatHelper::loginButtonText();
        $layers = FormatHelper::getLayers();
        $stepTwoText = FormatHelper::stepTwoButtonText();
        return view('admin.security.auth.login', compact('text', 'layers', 'stepTwoText'));
    }

    public function login(Request $request)
    {
        $result = $this->loginUser($request);
        return response()->json($result);
    }

    public function loginStepTwo(Request $request)
    {
        $result = $this->loginUserStepTwo($request);
        return response()->json($result);
    }

    public function loginStepThree(Request $request)
    {
        $result = $this->loginUserStepThree($request);
        return response()->json($result);
    }

    public function logout(Request $request)
    {
        //$this->logOutUser($request);
    }

    public function noPermission(Request $request)
    {
        return view('import.errors.access_denined');
    }

    public function changePassword(Request $request)
    {
        $input = $request->all();
        $resetInfo = $this->getResetInfo($input['change_token']);
        return view('admin.security.auth.passwords.reset', compact('resetInfo'));
    }

    public function resetPassword(Request $request)
    {
        $result = $this->passwordReset($request);
        return response()->json($result);
    }

    public function getEmail(Request $request)
    {
        $resetInfo = [];
        return view('admin.security.auth.passwords.get_email', compact('resetInfo'));
    }

    public function sendResetEmail(Request $request)
    {
        $input = $request->all();

        $result = $this->sendPasswordResetLink($input);
        return response()->json($result);
    }
}
