<?php

namespace App\Http\Controllers\Admin\Security\Permission;


use App\Models\Maintenance;
use App\Repositories\CommonRepo;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Spatie\Permission\Models\Permission;
use Spatie\Permission\Models\Role;
use Validator;

class PermissionController extends Controller
{
    public function rolePermission()
    {
        $roles = Role::all();

        return view('admin.security.permission.role_permission')->with('roles', $roles);
    }

    public function editRolePermission($id)
    {
        $role = Role::findOrFail($id);
        $permission = Permission::get();
        $rolePermissions = \DB::table("role_has_permissions")->where("role_has_permissions.role_id", $id)
            ->pluck('role_has_permissions.permission_id', 'role_has_permissions.permission_id')
            ->all();

        return view('admin.security.permission.role_permission_edit', compact('role', 'permission', 'rolePermissions'));
    }

    public function updateRolePermission(Request $request, $id)
    {
        $input = $request->all();

        $role = Role::find($id);
        $role->syncPermissions($request->input('permission'));
        $this->clearCache();
        $json = [
            'type' => 'success',
            'redirect' => url('role_permission'),
            'message' => 'Successfully saved.'
        ];

        return response()->json($json);
    }

    public function createRolePermission()
    {
        $roles = Role::all();
        $permission = Permission::all();

        return view('admin.security.permission.role_permission_create', compact('roles', 'permission'));
    }

    public function storeRolePermission(Request $request)
    {
        $input = $request->all();
        $rules = [
            'name' => 'required|unique:roles',
        ];

        $validator = Validator::make($input, $rules);

        if ($validator->fails()) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => $validator->messages()->toArray()
            ];

            return response()->json($json);
        }

        $role = Role::create(['name' => $request->input('name')]);
        $role->syncPermissions($request->input('permission'));
        $this->clearCache();
        $json = [
            'type' => 'success',
            'redirect' => url('role_permission'),
            'message' => 'Successfully saved.'
        ];

        return response()->json($json);

    }

    public function clearCache()
    {
        \Illuminate\Support\Facades\Artisan::call('cache:clear');
        \Illuminate\Support\Facades\Artisan::call('view:clear');
        \Illuminate\Support\Facades\Artisan::call('route:clear');
        \Illuminate\Support\Facades\Artisan::call('config:clear');
    }

    public function downSite()
    {
        return view('admin.security.permission.down_site');
    }

    public function suspend(Request $request)
    {
        $data = [
            'user_id' => Auth::id()
        ];
        CommonRepo::commonStore('App\Models\Maintenance', $data);
        $json = [
            'type' => 'success',
            'redirect' => 'reload',
            'message' => 'Application has successfully suspended.'
        ];
        return response()->json($json);
    }

    public function checkValid(Request $request)
    {
        $maintenance = Maintenance::all();
        if (!empty($maintenance) && count($maintenance) > 0) {
            $json = [
                'type' => 'success',
                'redirect' => 'reload',
                'message' => ''
            ];
        }

        return response()->json($json);
    }

    public function destroyRolePermission($id)
    {
        $role = Role::findById($id);

        $allPermission = Permission::all();

        foreach ($allPermission as $p) {
            $role->revokePermissionTo($p);
        }

        $json = [
            'type' => 'success',
            'redirect' => url('role_permission'),
            'message' => 'Successfully role permissions removed.'
        ];

        return response()->json($json);
    }
}
