<?php

namespace App\Http\Controllers\Admin\Security\User;


use App\Mail\SendMail;
use App\Repositories\CommonRepo;
use App\Repositories\UserRepo;
use App\User;
use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Spatie\Permission\Models\Role;
use App\Traits\FilterTrait;
use App\Traits\UserTrait;
use Yajra\DataTables\DataTables;


class UserController extends Controller
{
    use UserTrait;

    private $userRepo;

    public function __construct()
    {
        $this->userRepo = new UserRepo();
//        $this->middleware('permission:update', ['only' => ['edit','update']]);
//        $this->middleware('permission:list', ['only' => ['index']]);
//        $this->middleware('permission:create', ['only' => ['create', 'store']]);


    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $user = Auth::user();
        $userRole = $user->roles->all();
        if(isset($userRole[0]) && $userRole[0]->name != 'Super Admin'){
            return redirect('/');;
        }
        //dd($user->getDirectPermissions());
        //$permissions = $user->permissions;//dd($permissions);
        //        $user = Auth::user();
//        //$roles = Role::pluck('name','name')->all();

        //dd($userRole[0]->name);
        //dd($user);

        return view('admin.security.user.index');
    }

    public function getUserData(Request $request)
    {
        $input = $request->all();
        $userData = $this->userRepo->getAllUsers($input);

        return $this->userInfo($userData);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $user = Auth::user();
        $userRole = $user->roles->all();
        if(isset($userRole[0]) && $userRole[0]->name != 'Super Admin'){
            return redirect('/');;
        }

        $roles = $this->getRoles();
        $layers = CommonRepo::getActiveLayers();

        return view('admin.security.user.create', compact('roles', 'layers'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $input = $request->all();
        $validator = $this->userValidation($input);
        if ($validator->fails()) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => $validator->messages()->toArray()
            ];

            return response()->json($json);
        }

        $result = $this->saveUser($request);

        return response()->json($result);
    }

    /**
     * Display the specified resource.
     *
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $user = Auth::user();
        $userRole = $user->roles->all();
        if(isset($userRole[0]) && $userRole[0]->name != 'Super Admin'){
            return redirect('/');;
        }

        $roles = $this->getRoles();
        $layers = CommonRepo::getActiveLayers();
        $rows = CommonRepo::getAllById('App\User', $id);

        return view('admin.security.user.edit', compact('roles', 'layers', 'rows'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request $request
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $input = $request->all();
        $validator = $this->userValidationUpdate($input, $id);
        if ($validator->fails()) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => $validator->messages()->toArray()
            ];

            return response()->json($json);
        }

        $result = $this->updateUser($request, $id);

        return response()->json($result);
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $updateData = [
            'is_deleted' => 1,
            'deleted_at' => Carbon::parse()->toDateTimeString()
        ];

        User::where('id', $id)->update($updateData);

        $json = [
            'type' => 'success',
            'redirect' => url('user'),
            'message' => 'Successfully user deleted.'
        ];

        return response()->json($json);
    }

    public function validateUserName(Request $request)
    {
        $result = $this->checkUserName($request);
        return response()->json($result);
    }

//    public function sendToFriend(Request $request)
//    {
//        $input = $request->all();
//
//        //$data = view('web.emails.to_friend', compact('input'));
//        //$subject = "Vacancy on megajobs.lk - " . $input['vacancy'];
//        $subject = "Vacancy on megajobs.lk - ";
//        //$replyTo = $input['senderEmail'];
//
//        try {
//            Mail::to('pasan@pragicts.com')->send(new SendMail('', $subject));
//            $json = [
//                'megajobs' => true,
//                'message' => 'The vacancy has been successfully sent.',
//                'status' => 1
//            ];
//
//            return response()->json($json);
//        } catch (\Exception $e) {
//            dd($e);
//            $json = [
//                'megajobs' => false,
//                'message' => 'Sorry! Email has not been sent!',
//                'status' => 0
//            ];
//
//            return response()->json($json);
//        }
//
//    }
}
