<?php

namespace App\Traits\Api;

use App\Repositories\Api\ApiRepo;
use App\Repositories\CommonRepo;

trait  DataMigrationTrait
{
    private $apiRepo;

    public function __construct()
    {
        $this->apiRepo = new ApiRepo();
    }

    protected function sendItemToSell($sendData)
    {
        $token = $this->getTokenFromDb();
        $method = "POST";
        $url = env('API_BASE_URL_V1') . 'save_new_item';

        $result = $this->commonCall($token->access_token, $method, $url, $sendData);

        if ($result) {
            return true;
        }

        return false;
    }

    protected function updateItemToSell($sendData)
    {
        $token = $this->getTokenFromDb();
        $method = "POST";
        $url = env('API_BASE_URL_V1') . 'update_new_item';

        $result = $this->commonCall($token->access_token, $method, $url, $sendData);

        if ($result) {
            return true;
        }

        return false;
    }

    protected function deleteItemFromSell($sendData)
    {
        $token = $this->getTokenFromDb();
        $method = "POST";
        $url = env('API_BASE_URL_V1') . 'delete_item';

        $result = $this->commonCall($token->access_token, $method, $url, $sendData);

        if ($result) {
            return true;
        }

        return false;
    }

    protected function getItemFromSell()
    {
        $token = $this->getTokenFromDb();
        $method = "POST";
        $url = env('API_BASE_URL_V1') . 'get_item_from_sell';

        $sendData = [];

        $result = $this->commonCall($token->access_token, $method, $url, $sendData);

        if ($result) {

            if (isset($result['data']) && count($result['data']) > 0) {

                foreach ($result['data'] as $data) {
                    $whereArray = [
                        'it_agc_reference_no' => $data['it_agc_reference_no']
                    ];

                    $updateArray = [
                        'agc_update' => 1
                    ];

                    CommonRepo::commonUpdate('App\Models\Item', $whereArray, $updateArray);
                }

                $json = [
                    'type' => 'success',
                    'redirect' => 'reload',
                    'message' => 'Successfully item status has updated.',
                ];

                return response()->json($json);
            }

            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => 'No new items.',
            ];

            return response()->json($json);

        }

        $json = [
            'type' => 'error',
            'redirect' => url('connect_to_sell'),
            'message' => 'Please try again.',
        ];

        return response()->json($json);
    }

    protected function getNewOrder()
    {
        $token = $this->getTokenFromDb();
        $method = "POST";
        $url = env('API_BASE_URL_V1') . 'get_new_order';

        $result = $this->commonCall($token->access_token, $method, $url, []);
//dd($result);
        if ($result) {

            if (isset($result['data']) && count($result['data']) > 0) {
//                \DB::beginTransaction();
//
//                try {
                    foreach ($result['data'] as $data) {

                        $dataArray = [

                            'order_number' => $data['order_number'],
                            'item_category' => $data['item_category'],
                            //'item_category' => $data['item_category'],
                            'created_by' => $data['created_by'],
                            'created_at' => $data['created_at'],
                            'ship_date' => $data['ship_date']
                        ];

                        $order = CommonRepo::commonStore('App\Models\Order', $dataArray);

                        if (isset($data['order_detail'])) {
                            foreach ($data['order_detail'] as $val) {
                                $detailArray = [
                                    'order_id' => $order->id,
                                    'item_id' => $val['item_id'],
                                    'it_agc_reference_no' => $val['it_agc_reference_no'],
                                    'item_description' => $val['item_description'],
                                    'item_category' => $val['item_category'],
                                    'it_agc_measurement_unit_selling_point' => $val['it_agc_measurement_unit_selling_point'],
                                    'item_qty' => $val['item_qty'],
                                    'item_description_excel' => $val['item_description_excel'],
                                    'it_agc_measurement_unit_selling_point_excel' => $val['it_agc_measurement_unit_selling_point_excel']
                                ];

                                CommonRepo::commonStore('App\Models\OrderDetail', $detailArray);
                            }
                        }
                    }
//                } catch (\Exception $e) {
//                    \DB::rollback();
//                    //dd($e);
//                    $json = [
//                        'type' => 'error',
//                        'redirect' => '',
//                        'message' => 'Sorry! Please try again'
//                    ];
//
//                    return $json;
//                }

                //\DB::commit();

                $json = [
                    'type' => 'success',
                    'redirect' => url('order'),
                    'message' => 'Successfully order has received.',
                ];

                return response()->json($json);
            }

            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => 'No new orders.',
            ];

            return response()->json($json);
        }

        $json = [
            'type' => 'error',
            'redirect' => 'reload',
            'message' => 'Please try again.',
        ];

        return response()->json($json);
    }

}