<?php

namespace App\Traits;


use App\Helpers\FormatHelper;
use App\Models\Invoice;
use App\Repositories\CommonRepo;
use App\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Yajra\DataTables\DataTables;
use Validator;
use GuzzleHttp\Client;

trait InvoiceTrait
{
    use CommonTrait;

    public function sendSaveInvoice($request, $isSend)
    {
        $input = $request->all();

        $validator = $this->invoiceValidation($input);
        if ($validator->fails()) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => $validator->messages()->toArray()
            ];

            return response()->json($json);
        }

        $input['created_by'] = Auth::id();
        $resetToken = md5(str_random(10)) . md5(FormatHelper::randNumberGenerator(15));
        $input['token'] = $resetToken;

        try {
            $inv = CommonRepo::commonStore('App\Models\Invoice', $input);

            $dt = Carbon::now();

            if ($isSend == 1) {
                //dd($a);
                $payUrl = url('land_pay?pay_token=' . $resetToken);

                $user = Auth::user();
                $data = view('admin.invoice.email.pay_invoice_link_email', compact('input', 'payUrl', 'user'));
                $subject = "Authenticities Payment Platform";
                $message = 'Successfully invoice sent';

                // send email with payable
                //$result = $this->sendCommonMail($input['email'], $data, $subject, $message);
                $result = $this->sendCommonMailInvoice($input['email'], $data, $subject, $message, $user->email);
                if ($result['step'] == 3) {
                    CommonRepo::commonUpdate('App\Models\Invoice', ['id' => $inv->id], ['is_send' => 1, 'token_send_at' => $dt->toDateString()]);

                    $json = [
                        'type' => 'success',
                        'redirect' => 'reload',
                        'step' => 0,
                        'message' => 'Successfully invoice saved and sent.'
                    ];

                    return $json;
                }
            }

            $json = [
                'type' => 'success',
                'redirect' => 'reload',
                'step' => 0,
                'message' => 'Successfully invoice saved but not sent'
            ];

            return $json;
        } catch (\Exception $e) {
            //dd($e);
            $json = [
                'type' => 'error',
                'redirect' => 'reload',
                'step' => 0,
                'message' => 'Sorry! Please try again.'
            ];

            return $json;
        }

        return response()->json($json);

    }

    private function getInvoiceById($id)
    {
        return Invoice::select('id', 'name', 'tour_name', 'bfn_number', 'email', 'amount', 'description', 'invoice_date', 'is_paid')
            ->where('id', $id)->first();
    }

    public function updateInvoice($id, $input)
    {
        $input['updated_by'] = Auth::id();
        $resetToken = md5(str_random(10)) . md5(FormatHelper::randNumberGenerator(15));
        $input['token'] = $resetToken;

        try {

            CommonRepo::commonUpdate('App\Models\Invoice', ['id' => $id], $input);

            $dt = Carbon::now();

            //dd($a);
            $payUrl = url('land_pay?pay_token=' . $resetToken);

            $user = Auth::user();
            $data = view('admin.invoice.email.pay_invoice_link_email', compact('input', 'payUrl', 'user'));
            $subject = "Authenticities Payment Platform";
            $message = 'Successfully invoice sent';

            // send email with payable
            //$result = $this->sendCommonMail($input['email'], $data, $subject, $message);
            $result = $this->sendCommonMailInvoice($input['email'], $data, $subject, $message, $user->email);
            if ($result['step'] == 3) {
                CommonRepo::commonUpdate('App\Models\Invoice', ['id' => $id], ['is_send' => 1, 'token_send_at' => $dt->toDateString()]);
                $result['message'] = 'Successfully invoice saved and sent.';
                return $result;
            }

            $result['message'] = 'Successfully invoice saved but not sent';
            return $result;
        } catch (\Exception $e) {
            //dd($e);
            $json = [
                'type' => 'error',
                'redirect' => 'reload',
                'step' => 0,
                'message' => 'Sorry! Please try again.'
            ];

            return $json;
        }

        return response()->json($json);
    }

    public function getInvoice($request)
    {
        $user = Auth::user();
        $userRole = $user->roles->all();
//dd($userRole);
        $data = Invoice::select('id',
            'tour_name', 'invoice_date', 'bfn_number', 'email', 'amount', 'is_paid', 'is_send'
        )->where(function ($query) use ($request, $userRole, $user) {
            if (isset($request->tour_name) && $request->tour_name != "") {
                $query->Where('tour_name', 'like', '%' . $request->tour_name . '%');
            }
            if (isset($request->bfn_number) && $request->bfn_number != "") {
                $query->Where('bfn_number', $request->bfn_number);
            }
            if (isset($request->email) && $request->email != "") {
                $query->Where('email', $request->email);
            }
            if (isset($request->is_paid) && $request->is_paid != "3") {
                $query->Where('is_paid', $request->is_paid);
            }
            if (isset($userRole[0]) && ($userRole[0]->name != 'Management' && $userRole[0]->name != 'Super Admin')) {
                $query->Where('created_by', $user->id);
            }
        })->orderBy('id', 'desc')->get();


        return DataTables::of($data)
            ->addIndexColumn()
            ->editColumn('is_send', function ($data) {

                if ($data->is_send == 1) {
                    $sendStatus = '<div class="badge border-success success badge-border">Sent</div>';
                } else {
                    $sendStatus = '<div class="badge border-danger danger badge-border">Not Sent</div>';
                }
                return $sendStatus;
            })
            ->editColumn('is_paid', function ($data) {

                if ($data->is_paid == 1) {
                    $status = '<div class="badge border-success success badge-border">Paid</div>';
                } else {
                    $status = '<div class="badge border-danger danger badge-border">Not Paid</div>';
                }
                return $status;
            })
            ->addColumn('action', function ($data) {

                $action = '<div class="btn-group mr-1 mb-1">
									<button type="button" class="btn btn-info dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                    Action
									</button>
									<div class="dropdown-menu arrow">';

                if ($data->is_paid != 1) {
                    $action .= '<a target="_blank" href="' . url('invoices/' . $data->id . '/edit?send=false') . '" style="cursor: pointer" class="dropdown-item">Edit</a>';
                    //$action .= '<a target="_blank" href="' . url('invoices/' . $data->id . '/edit?send=true') . '" style="cursor: pointer" class="dropdown-item">Send</a>';
                } else {
                    $action .= '<a target="_blank" href="' . url('invoices/' . $data->id . '/edit') . '" style="cursor: pointer" class="dropdown-item">View</a>';
                }
                if ($data->is_send != 1) {
                    $action .= '<button style="cursor: pointer" class="dropdown-item delete-btn" type="button" data-delete_url="' . url('delete_invoice/' . $data->id) . '">Delete</button>																	
									</div>
								</div>';
                }
                //$deleteText = '<button class="btn btn-outline-danger delete-btn btn-sm" data-delete_url="' . url('item_delete/' . $data->it_id) . '"><i class="la la-trash-o"></i> </button> ';
                //$lastText = '<button class="btn btn-outline-warning delete-btn btn-sm" data-delete_url="' . url('item_delete/' . $data->it_id) . '"><i class="la la-paper-plane"></i> </button>';


                return $action;

            })
            ->rawColumns(['action', 'is_send', 'is_paid'])
            ->make(true);
    }

    protected function sendEmailAgain($request)
    {
        $input = $request->all();
    }

    protected function deleteInvoice($id)
    {
        CommonRepo::commonDelete('App\Models\Invoice', ['id' => $id]);
        $json = [
            'type' => 'success',
            'redirect' => url('invoice'),
            'message' => 'Successfully invoice deleted.',
        ];
        return response()->json($json);

    }

    public function callIPG($request)
    {
        $input = $request->all();
        //dd($input);
        $id = \Crypt::decryptString($input['inv_ref']);


        $invoiceInfo = $this->getInvoiceById($id);

        if (empty($invoiceInfo)) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => 'Invalid information. Please contact head office.'
            ];

            return response()->json($json);
        }

        if ($invoiceInfo->is_paid) {
            $json = [
                'type' => 'error',
                'redirect' => url('notify'),
                'message' => 'Thank you. You have already settled this invoice.'
            ];

            return response()->json($json);
        }

        $input['id'] = $invoiceInfo->id;
        $input['tour_name'] = $invoiceInfo->tour_name;
        $input['bfn_number'] = $invoiceInfo->bfn_number;
        $input['email'] = $invoiceInfo->email;
        $input['amount'] = $invoiceInfo->amount;

        $input['card_number'] = str_replace(" ", "", trim($input['card_number']));

        $validator = $this->cardValidation($input);

        if ($validator->fails()) {
            $json = [
                'type' => 'error',
                'redirect' => '',
                'message' => $validator->messages()->toArray()
            ];

            return response()->json($json);
        }

        return $this->sendInfo($input);
    }


    private function sendInfo($input)
    {
        try {

            $client = new Client([
                'base_uri' => env('CYBE_CALL_BASE_URL'),
                'headers' => [
                    'Accept' => 'application/json',
                    'Content-Type' => 'application/json',
                ],
            ]);

            $dataArray = [
                'form_params' => [
                    'cyber_call_key' => env('CYBER_CALL_KEY'),
                    'id' => $input['id'],
                    'tour_name' => $input['tour_name'],
                    'bfn_number' => $input['bfn_number'],
                    'email' => $input['email'],
                    'amount' => $input['amount'],
                    'card_number' => $input['card_number'],
                    'month' => $input['month'],
                    'year' => $input['year'],
                    'cvv_number' => $input['cvv_number'],
                ]
            ];

            //$response = $client->request("POST", 'Samples/Payments/CoreServices/ProcessPayment.php', $dataArray);
            $response = $client->request("POST", 'https://authenticitiessrilanka.com/authen/cs/Samples/Payments/CoreServices/ProcessPayment.php', $dataArray);
            //dd($response->getBody()->getContents());
            $res = json_decode($response->getBody()->getContents());
//dd($res);
            if ($res->status == 'error') {
                $json = [
                    'type' => 'error',
                    'redirect' => 'reload',
                    'message' => $res->message
                ];
                return $json;
            }

            //if ($res->status == 'success') {
            $json = [
                'type' => 'success',
                'redirect' => url('notify?trn_id=' . $res->trn_id),
                'message' => $res->message
            ];
            return $json;
            //}

        } catch (\Exception $exception) {
            //dd($exception->getMessage());
            $json = [
                'type' => 'error',
                'redirect' => 'reload',
                'message' => 'Sorry! Please try again.'
            ];
            \Log::info($exception->getMessage());
            return $json;

        }

        return response()->json($json);

    }

    private function cardValidation($input)
    {
        $rules = [
            'month' => 'required|min:2',
            'card_number' => ['required', 'numeric'],
            'year' => 'required|min:4',
            'cvv_number' => 'required|max:4'
        ];

        return Validator::make($input, $rules);
    }

    private function invoiceValidation($input)
    {
        $rules = [
            'name' => ['required', 'regex:/(^[A-Za-z0-9 ]+$)+/'],
            'tour_name' => ['required', 'regex:/(^[A-Za-z0-9 ]+$)+/']
        ];

        $message = [
            'name' => 'Please use numbers and letters for name field',
            'tour_name' => 'Please use numbers and letters for name tour name field'
        ];

        return Validator::make($input, $rules, $message);
    }

    protected function checkTokenValid($input)
    {
        $token = isset($input['pay_token']) ? $input['pay_token'] : "";
        $setting = FormatHelper::settingInfo();
        $invoiceInfo = Invoice::select('id', 'token', 'token_send_at', 'is_paid')->where('token', $token)->get();

        if ($invoiceInfo->isEmpty()) {
            return 1;//invalid token
        }

        $expireDate = Carbon::parse($invoiceInfo[0]->token_send_at)
            ->addDays($setting->token_exp_date_count);


        if ($invoiceInfo[0]->token != $token) {
            return 2;//invalid token
        }

        $today = Carbon::now()->toDateString();

        if (!$expireDate->greaterThan($today)) {
            return 3;//token expired
        }

        if ($invoiceInfo[0]->is_paid == 1) {
            return 4;//paid
        }

        return 5;//valid token
    }

    protected function getInvoiceInfo($token)
    {
        return Invoice::select('id', 'tour_name', 'bfn_number', 'email', 'amount', 'invoice_date')
            ->where('token', $token)->first();
    }

    protected function decodePayInfo($request)
    {
        $input = $request->all();
        $invoiceInfo = Invoice::select('id', 'payment_info')
            ->where('id', $input['id'])->first();//dd($invoiceInfo);
        dd(unserialize(base64_decode($invoiceInfo->payment_info)));
    }

}