<?php

namespace App\Traits;


use App\Helpers\FormatHelper;
use App\Models\Invoice;
use App\Repositories\CommonRepo;
use App\User;
use Carbon\Carbon;
use Illuminate\Support\Facades\Auth;
use Yajra\DataTables\DataTables;
use Validator;
use GuzzleHttp\Client;
use Zend\Diactoros\Request;

trait ReportTrait
{
    use CommonTrait;

    public function getInvoice($request)
    {
        $user = Auth::user();
        $userRole = $user->roles->all();
//dd($userRole);
        $data = Invoice::select('id',
            'tour_name', 'invoice_date', 'bfn_number', 'email', 'amount', 'is_paid', 'is_send', 'invoice_date', 'request_id'
        )->where(function ($query) use ($request, $userRole, $user) {
            if (isset($request->tour_name) && $request->tour_name != "") {
                $query->Where('tour_name', 'like', '%' . $request->tour_name . '%');
            }
            if (isset($request->bfn_number) && $request->bfn_number != "") {
                $query->Where('bfn_number', $request->bfn_number);
            }
            if (isset($request->email) && $request->email != "") {
                $query->Where('email', $request->email);
            }
            if (isset($request->is_paid) && $request->is_paid != "3") {
                $query->Where('is_paid', $request->is_paid);
            }

            if (isset($request->to_date) && $request->to_date != "") {
                $query->Where('invoice_date', '<=', $request->to_date);
            }
            if (isset($request->from_date) && $request->from_date != "") {
                $query->Where('invoice_date', '>=', $request->from_date);
            }

            if (isset($userRole[0]) && ($userRole[0]->name != 'Management' && $userRole[0]->name != 'Super Admin')) {
                $query->Where('created_by', $user->id);
            }
        })->orderBy('id', 'desc')->get();


        return DataTables::of($data)
            ->addIndexColumn()
            ->editColumn('is_send', function ($data) {

                if ($data->is_send == 1) {
                    $sendStatus = '<div class="badge border-success success badge-border">Sent</div>';
                } else {
                    $sendStatus = '<div class="badge border-danger danger badge-border">Not Sent</div>';
                }
                return $sendStatus;
            })
            ->editColumn('is_paid', function ($data) {

                if ($data->is_paid == 1) {
                    $status = '<div class="badge border-success success badge-border">Paid</div>';
                } else {
                    $status = '<div class="badge border-danger danger badge-border">Not Paid</div>';
                }
                return $status;
            })
            ->editColumn('request_id', function ($data) {

                if ($data->request_id == "") {
                    $trn = '-';
                } else {
                    $trn = $data->request_id;
                }
                return $trn;
            })
            ->addColumn('action', function ($data) {

                $action = '<div class="btn-group mr-1 mb-1">
									<button type="button" class="btn btn-info dropdown-toggle" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
                    Action
									</button>
									<div class="dropdown-menu arrow">';

                if ($data->is_paid != 1) {
                    $action .= '<a target="_blank" href="' . url('invoices/' . $data->id . '/edit?send=false') . '" style="cursor: pointer" class="dropdown-item">Edit</a>';
                    //$action .= '<a target="_blank" href="' . url('invoices/' . $data->id . '/edit?send=true') . '" style="cursor: pointer" class="dropdown-item">Send</a>';
                } else {
                    $action .= '<a target="_blank" href="' . url('invoices/' . $data->id . '/edit') . '" style="cursor: pointer" class="dropdown-item">View</a>';
                }
                if ($data->is_send != 1) {
                    $action .= '<button style="cursor: pointer" class="dropdown-item delete-btn" type="button" data-delete_url="' . url('delete_invoice/' . $data->id) . '">Delete</button>																	
									</div>
								</div>';
                }
                //$deleteText = '<button class="btn btn-outline-danger delete-btn btn-sm" data-delete_url="' . url('item_delete/' . $data->it_id) . '"><i class="la la-trash-o"></i> </button> ';
                //$lastText = '<button class="btn btn-outline-warning delete-btn btn-sm" data-delete_url="' . url('item_delete/' . $data->it_id) . '"><i class="la la-paper-plane"></i> </button>';


                return $action;

            })
            ->rawColumns(['action', 'is_send', 'is_paid', 'request_id'])
            ->make(true);
    }

}